@@ -6,7 +6,7 @@ from IPython.core.magic import (
     line_cell_magic,
 )
 
-from icortex.kernel import get_icortex_kernel, print_service_help
+from icortex.kernel import get_icortex, print_service_help
 
 
 @magics_class
@@ -16,10 +16,28 @@ class ICortexMagics(Magics):
         print_service_help()
         return
 
+    @line_magic
+    def var(self, line):
+        "Define a variable"
+        shell = get_icortex()
+        return shell.eval_var(line)
+
+    @line_magic
+    def export(self, line):
+        "Export ICortex notebook"
+        shell = get_icortex()
+        return shell.export(line)
+
+    @line_magic
+    def bake(self, line):
+        "Bake ICortex notebook"
+        shell = get_icortex()
+        return shell.bake(line)
+
     @line_magic
     def icortex(self, line):
-        kernel = get_icortex_kernel()
-        return kernel.cli(line)
+        shell = get_icortex()
+        return shell.cli(line)
 
     @line_cell_magic
     def prompt(self, line, cell=None):
@@ -34,8 +52,8 @@ class ICortexMagics(Magics):
         return self.prompt(line, cell=cell)
 
     def _prompt(self, input_):
-        kernel = get_icortex_kernel()
-        return kernel.prompt(input_)
+        shell = get_icortex()
+        return shell.prompt(input_)
 
 
 def load_ipython_extension(ipython):
